@@ -38,14 +38,15 @@ FAST_TEST = False #: determines whether we skip slow tests or not
 # This list also determines the order in which tests will executed
 __all__ = [
 	'package', 'translations',
-	'errors', 'parsing', 'fs', 'config', 'applications', 'async',
+	'utils', 'errors', 'parsing', 'fs', 'config', 'applications', 'async',
 	'formats', 'templates',
-	'stores', 'index', 'notebook',
-	'history', 'plugins',
+	'stores', 'index', 'notebook', 'history',
+	'main', 'plugins',
 	'export', 'www', 'search',
-	'widgets', 'gui', 'pageview',
+	'widgets', 'gui', 'pageview', 'clipboard',
 	'calendar', 'printtobrowser', 'versioncontrol', 'inlinecalculator',
-	'tasklist', 'tags', 'imagegenerators',
+	'tasklist', 'tags', 'imagegenerators', 'tableofcontents',
+	'quicknote', 'attachmentbrowser',
 	'daemon' # Note that running this test in another position can skrew up e.g. clipboard test
 ]
 
@@ -124,15 +125,6 @@ def zim_pyfiles():
 		yield file # shallow copy
 
 
-
-def gtk_process_events(*a):
-	'''Method to simulate a few iterations of the gtk main loop'''
-	import gtk
-	while gtk.events_pending():
-		gtk.main_iteration(block=False)
-	return True # continue
-
-
 def slowTest(obj):
 	'''Decorator for slow tests
 
@@ -164,6 +156,22 @@ def slowTest(obj):
 class TestCase(unittest.TestCase):
 	'''Base class for test cases'''
 
+	maxDiff = None
+
+	def assertEqual(self, first, second, msg=None):
+		## HACK to work around bug in unittest - it does not consider
+		## string and unicode to be of the same type and thus does not
+		## show diffs
+		## TODO file bug report for this
+		if type(first) in (str, unicode) \
+		and type(second) in (str, unicode):
+			self.assertMultiLineEqual(second, first, msg)
+			## HACK switch arguments here, otherwise order of
+			## diff is wrong (assuming first is what we got and second
+			## is the reference)
+		else:
+			unittest.TestCase.assertEqual(self, first, second, msg)
+
 	def create_tmp_dir(self, name=None):
 		'''Returns a path to a tmp dir where tests can write data.
 		The dir is removed and recreated empty every time this function
@@ -226,7 +234,11 @@ class LoggingFilter(object):
 		self.loggerobj.removeFilter(self)
 
 	def filter(self, record):
-		return not record.getMessage().startswith(self.message)
+		msg = record.getMessage()
+		if isinstance(self.message, tuple):
+			return not any(msg.startswith(m) for m in self.message)
+		else:
+			return not msg.startswith(self.message)
 
 	def wrap_test(self, test):
 		self.__enter__()
@@ -339,6 +351,49 @@ def _expand_manifest(names):
 	return manifest
 
 
+def new_parsetree():
+	'''Returns a new ParseTree object for testing
+
+	Uses data from L{WikiTestData}, page C{roundtrip}
+	'''
+	import zim.formats.wiki
+	parser = zim.formats.wiki.Parser()
+	text = WikiTestData.get('roundtrip')
+	tree = parser.parse(text)
+	return tree
+
+def new_parsetree_from_text(text, format='wiki'):
+	import zim.formats
+	parser = zim.formats.get_format(format).Parser()
+	return parser.parse(text)
+
+
+def new_parsetree_from_xml(xml):
+	# For some reason this does not work with cElementTree.XMLBuilder ...
+	from xml.etree.ElementTree import XMLTreeBuilder
+	from zim.formats import ParseTree
+	builder = XMLTreeBuilder()
+	builder.feed(xml)
+	root = builder.close()
+	return ParseTree(root)
+
+
+def new_page():
+	from zim.notebook import Path, Page
+	page = Page(Path('roundtrip'))
+	page.readonly = False
+	page.set_parsetree(new_parsetree())
+	return page
+
+
+def new_page_from_text(text, format='wiki'):
+	from zim.notebook import Path, Page
+	page = Page(Path('Test'))
+	page.readonly = False
+	page.set_parsetree(new_parsetree_from_text(text, format))
+	return page
+
+
 def new_notebook(fakedir=None):
 	'''Returns a new Notebook object with all data in memory
 
@@ -413,25 +468,18 @@ class Counter(object):
 		self.count += 1
 		return self.value
 
+class MockObjectBase(object):
+	'''Base class for mock objects.
 
-class MockObject(object):
-	'''Base class for mock objects. Any attribute is automatically
-	vivicated as a method which results None. Alternatively mock
-	methods can be installed with mock_method(). Attributes that are
-	not methods need to be initialized explicitly.
-
-	All method call are logged, so they can be inspected. The attribute
-	'mock_calls' has a list of tuples with mock methods and arguments in
-	order they have been called.
+	Mock methods can be installed with L{mock_method()}. All method
+	calls to mock methods are logged, so they can be inspected.
+	The attribute C{mock_calls} has a list of tuples with mock methods
+	and arguments in order they have been called.
 	'''
 
 	def __init__(self):
 		self.mock_calls = []
 
-	def __getattr__(self, name):
-		'''Automatically mock methods'''
-		return self.mock_method(name, None)
-
 	def mock_method(self, name, return_value):
 		'''Installs a mock method with a given name that returns
 		a given value.
@@ -445,3 +493,50 @@ class MockObject(object):
 
 		setattr(self, name, my_mock_method)
 		return my_mock_method
+
+
+class MockObject(MockObjectBase):
+	'''Simple subclass of L{MockObjectBase} that automatically mocks a
+	method which returns C{None} for any non-existing attribute.
+	Attributes that are not methods need to be initialized explicitly.
+	'''
+
+	def __getattr__(self, name):
+		'''Automatically mock methods'''
+		return self.mock_method(name, None)
+
+
+def gtk_process_events(*a):
+	'''Method to simulate a few iterations of the gtk main loop'''
+	import gtk
+	while gtk.events_pending():
+		gtk.main_iteration(block=False)
+	return True # continue
+
+
+def gtk_get_menu_item(menu, id):
+	'''Get a menu item from a C{gtk.Menu}
+	@param menu: a C{gtk.Menu}
+	@param id: either the menu item label or the stock id
+	@returns: a C{gtk.MenuItem} or C{None}
+	'''
+	items = menu.get_children()
+	ids = [i.get_property('label') for i in items]
+		# gtk.ImageMenuItems that have a stock id happen to use the
+		# 'label' property to store it...
+
+	assert id in ids, \
+		'Menu item "%s" not found, we got:\n' % id \
+		+ ''.join('- %s \n' % i for i in ids)
+
+	i = ids.index(id)
+	return items[i]
+
+
+def gtk_activate_menu_item(menu, id):
+	'''Trigger the 'click' action an a menu item
+	@param menu: a C{gtk.Menu}
+	@param id: either the menu item label or the stock id
+	'''
+	item = gtk_get_menu_item(menu, id)
+	item.activate()
