@@ -1,10 +1,9 @@
-
 # -*- coding: utf-8 -*-
 
 # Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 import tests
-from tests import TestCase
+
 
 from datetime import date as dateclass
 
@@ -13,9 +12,9 @@ from zim.notebook import Path
 from zim.config import ConfigDict
 
 
-class TestCalendarPlugin(TestCase):
+@tests.slowTest
+class TestCalendarPlugin(tests.TestCase):
 
-	slowTest = True
 
 	def runTest(self):
 		'Test Calendar plugin'
@@ -38,11 +37,11 @@ class StubUI(object):
 	ui_type = 'stub'
 
 	def __init__(self):
-		self.notebook = tests.get_test_notebook()
+		self.notebook = tests.new_notebook()
 		self.page = self.notebook.get_page(Path('Test:foo'))
 		self.preferences = ConfigDict()
 		self.uistate = ConfigDict()
-	
+
 	def connect(*a): pass
 
 	def connect_after(*a): pass
