@@ -6,159 +6,48 @@ from __future__ import with_statement
 
 import tests
 
+import gtk
 
 from zim.errors import Error
 from zim.notebook import get_notebook_list, Path, NotebookInfo
 from zim.fs import File, Dir
 from zim.config import config_file
+from zim.gui.clipboard import Clipboard
 
 import zim.gui
 
-from zim.gui.clipboard import *
 
+def setupGtkInterface(test, klass=None):
+	'''Setup a new GtkInterface object for testing.
+	Will have test notebook, and default preferences.
+	@param test: the test that wants to use this ui object
+	@param klass: the klass to use, defaults to L{GtkInterface}, but
+	could be partially mocked subclass
+	'''
+	if klass is None:
+		klass = zim.gui.GtkInterface
 
-@tests.slowTest
-class TestClipboard(tests.TestCase):
+	# start filtering
+	filter = FilterNoSuchImageWarning()
+	filter.wrap_test(test)
 
-	def runTest(self):
-		'''Test clipboard interaction'''
-		path = self.get_tmp_name()
-		notebook = tests.new_notebook(fakedir=path)
+	# flush preferences
+	preferences = config_file('preferences.conf')
+	preferences.file.remove()
 
-		clipboard = Clipboard()
+	# create interface object with new notebook
+	dirpath = test.get_tmp_name()
+	notebook = tests.new_notebook(fakedir=dirpath)
+	path = Path('Test:foo:bar')
+	ui = klass(notebook=notebook, page=path)
 
-		# tree roundtrip
-		for pagename in ('Test:wiki', 'roundtrip'):
-			page = notebook.get_page(Path(pagename))
-			parsetree = page.get_parsetree()
+	# finalize plugins
+	for plugin in ui.plugins:
+		plugin.finalize_ui(ui)
 
-			clipboard.set_parsetree(notebook, page, parsetree)
-			newtree = clipboard.request_parsetree(None, notebook, block=True)
-			self.assertEqual(newtree.tostring(), parsetree.tostring())
+	ui.mainwindow.init_uistate()
 
-		# tree -> ...
-		import zim.formats
-		text = 'some **bold** text'
-		parsetree = zim.formats.get_format('plain').Parser().parse(text.decode('utf-8'))
-		clipboard.set_parsetree(notebook, page, parsetree)
-
-		wanted = 'some **bold** text\n'
-		text = clipboard.wait_for_text()
-		self.assertEqual(text, wanted)
-
-		wanted = '''\
-<html>
-<head>
-<meta http-equiv="Content-Type" content="text/html;charset=utf-8">
-<meta name="Description" content="Copy-Paste Buffer">
-<meta name="Generator" content="Zim">
-</head>
-<body>
-<p>
-some <strong>bold</strong> text<br>
-</p>
-
-</body>
-</html>
-'''
-		selection = clipboard.wait_for_contents('text/html')
-		self.assertEqual(selection.data, wanted)
-
-		wanted = '''\
-Version:1.0\r
-StartHTML:000000185\r
-EndHTML:000000527\r
-StartFragment:000000450\r
-EndFragment:000000495\r
-StartSelection:000000450\r
-EndSelection:000000495\r
-SourceURL:zim://copy-pase-buffer\r
-<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN"><HTML><HEAD><meta http-equiv="Content-Type" content="text/html;charset=utf-8">
-<meta name="Description" content="Copy-Paste Buffer">
-<meta name="Generator" content="Zim"></HEAD><BODY><!--StartFragment--><p>
-some <strong>bold</strong> text<br>
-</p>
-<!--EndFragment--></BODY></HTML>'''
-		selection = clipboard.wait_for_contents('HTML Format')
-		self.assertEqual(selection.data, wanted)
-
-
-		# pagelink -> ..
-		page = notebook.get_page(Path('Test:wiki'))
-		clipboard.set_pagelink(notebook, page)
-
-		selection = clipboard.wait_for_contents(INTERNAL_PAGELIST_TARGET_NAME)
-		self.assertEqual(selection.data, 'Test:wiki\r\n')
-
-		selection = clipboard.wait_for_contents(PAGELIST_TARGET_NAME)
-		self.assertEqual(selection.data, 'Unnamed Notebook?Test:wiki\r\n')
-
-		wanted = '''\
-<?xml version=\'1.0\' encoding=\'utf-8\'?>\n<zim-tree><link href="+wiki">+wiki</link></zim-tree>'''
-		newtree = clipboard.request_parsetree(None, notebook, Path('Test'), block=True)
-		self.assertEqual(newtree.tostring(), wanted)
-
-		text = clipboard.wait_for_text()
-		self.assertEqual(text, 'Test:wiki')
-
-
-		# text -> tree
-		wanted = '''\
-<?xml version='1.0' encoding='utf-8'?>\n<zim-tree>some string</zim-tree>'''
-		clipboard.set_text('some string')
-		newtree = clipboard.request_parsetree(None, notebook, block=True)
-		self.assertEqual(newtree.tostring(), wanted)
-
-
-		# file -> tree
-		page = notebook.get_page(Path('Test:wiki'))
-		targets = [('text/uri-list', 0, 0)]
-
-		def my_get_data(clipboard, selectiondata, id, file):
-			selectiondata.set_uris([file.uri])
-
-		def my_clear_data(*a):
-			pass
-
-		file = File('/foo/bar/baz.txt')
-		clipboard.set_with_data(targets, my_get_data, my_clear_data, file)
-		tree = clipboard.request_parsetree(None, notebook, page, block=True)
-		link = tree.find('link')
-		rel_path = link.get('href')
-		self.assertEqual(notebook.resolve_file(rel_path, page), file)
-
-		file = File('./data/zim.png')
-		clipboard.set_with_data(targets, my_get_data, my_clear_data, file)
-		tree = clipboard.request_parsetree(None, notebook, page, block=True)
-		img = tree.find('img')
-		file_obj = img.get('_src_file')
-		self.assertEqual(file_obj, file)
-		rel_path = img.get('src')
-		self.assertEqual(notebook.resolve_file(rel_path, page), file)
-
-
-		# image -> tree
-		page = notebook.get_page(Path('Test:wiki'))
-		targets = [('image/png', 0, 0)]
-
-		def my_get_data(clipboard, selectiondata, id, file):
-			pixbuf = gtk.gdk.pixbuf_new_from_file(file.path)
-			selectiondata.set_pixbuf(pixbuf)
-
-		def my_clear_data(*a):
-			pass
-
-		file = File('./data/zim.png')
-		clipboard.set_with_data(targets, my_get_data, my_clear_data, file)
-		tree = clipboard.request_parsetree(None, notebook, page, block=True)
-		img = tree.find('img')
-		file_obj = img.get('_src_file')
-		self.assertFalse(file_obj == file)
-		self.assertTrue(file_obj.exists())
-		self.assertTrue(file_obj.isimage())
-		self.assertTrue(file_obj.path.endswith('.png'))
-		rel_path = img.get('src')
-		self.assertEqual(notebook.resolve_file(rel_path, page), file_obj)
+	return ui
 
 
 @tests.slowTest
@@ -404,6 +293,7 @@ class TestDialogs(tests.TestCase):
 			'icon': './icon.png',
 			'document_root': File('/foo').path, # win32 save test
 			'shared': False,
+			'profile': '',
 		}
 		config2 = {
 			'name': 'Notebook Bar',
@@ -412,6 +302,7 @@ class TestDialogs(tests.TestCase):
 			'icon': './picture.png',
 			'document_root': File('/bar').path, # win32 save test
 			'shared': True,
+			'profile': 'foo',
 		}
 		notebook.save_properties(**config1)
 		self.assertEqual(notebook.config['Notebook'], config1)
@@ -444,6 +335,7 @@ class TestDialogs(tests.TestCase):
 		gui = zim.gui.GtkInterface()
 		gui.register_preferences('GtkInterface', zim.gui.ui_preferences)
 		gui.register_preferences('PageView', zim.gui.pageview.ui_preferences)
+		gui.load_plugins()
 		self.ui.preferences_register = gui.preferences_register
 		self.ui.preferences = gui.preferences
 		self.ui.plugins = gui.plugins
@@ -499,23 +391,7 @@ class FilterNoSuchImageWarning(tests.LoggingFilter):
 class TestGtkInterface(tests.TestCase):
 
 	def setUp(self):
-		# start filtering
-		filter = FilterNoSuchImageWarning()
-		filter.wrap_test(self)
-
-		# flush preferences
-		preferences = config_file('preferences.conf')
-		preferences.file.remove()
-
-		# create interface object with new notebook
-		dirpath = self.get_tmp_name()
-		notebook = tests.new_notebook(fakedir=dirpath)
-		path = Path('Test:foo:bar')
-		self.ui = zim.gui.GtkInterface(notebook=notebook, page=path)
-
-		# finalize plugins
-		for plugin in self.ui.plugins:
-			plugin.finalize_ui(self.ui)
+		self.ui = setupGtkInterface(self)
 
 	def tearDown(self):
 		self.ui.close()
@@ -544,6 +420,11 @@ class TestGtkInterface(tests.TestCase):
 		items = menu.get_children()
 		self.assertGreater(len(items), 3)
 
+		# open notebook (so the default plugins are loaded)
+		nb = ui.notebook
+		ui.notebook = None
+		ui.open_notebook(nb)
+
 		# remove plugins
 		self.assertGreater(len(ui.plugins), 3) # default plugins
 		plugins = [p.plugin_key for p in ui.plugins]
@@ -696,6 +577,96 @@ class TestGtkInterface(tests.TestCase):
 	# TODO notebook manipulation (new (sub)page, move, rename, delete ..)
 	# merge with tests for dialogs (?)
 
+	def testClipboard(self):
+		self.ui.copy_location()
+		self.assertEqual(Clipboard.get_text(), 'Test:foo:bar')
+
+
+@tests.slowTest
+class TestClickLink(tests.TestCase):
+	'''Test to check pageview and GtkInterface play together nicely when
+	a link is clicked
+	'''
+
+	def setUp(self):
+		class MyMock(zim.gui.GtkInterface, tests.MockObjectBase):
+
+			def __init__(self, *arg, **kwarg):
+				zim.gui.GtkInterface.__init__(self, *arg, **kwarg)
+				tests.MockObjectBase.__init__(self)
+				for method in (
+					'open_file',
+					'open_with',
+					'open_notebook',
+					'open_page',
+				):
+					self.mock_method(method, None)
+
+		self.ui = setupGtkInterface(self, klass=MyMock)
+
+	def tearDown(self):
+		self.ui.close()
+
+	def runTest(self):
+		self.assertRaises(AssertionError, self.ui.open_url, 'foo@bar.com')
+			# this is not a URI, "mailto:foo@bar.com" is
+
+		# Note: same list of test uris is testing in tests.parsing as well
+		for href, type in (
+			('zim+file://foo/bar?dus.txt', 'notebook'),
+			('file:///foo/bar', 'file'),
+			('http://foo/bar', 'http'),
+			('http://192.168.168.100', 'http'),
+			('file+ssh://foo/bar', 'file+ssh'),
+			('mailto:foo@bar.com', 'mailto'),
+			('mailto:foo.com', 'page'),
+			('foo@bar.com', 'mailto'),
+			('mailto:foo//bar@bar.com', 'mailto'), # is this a valid mailto uri ?
+			('mid:foo@bar.org', 'mid'),
+			('cid:foo@bar.org', 'cid'),
+			('./foo/bar', 'file'),
+			('/foo/bar', 'file'),
+			('~/foo', 'file'),
+			('C:\\foo', 'file'),
+			('wp?foo', 'interwiki'),
+			('http://foo?bar', 'http'),
+			('\\\\host\\foo\\bar', 'smb'),
+			('foo', 'page'),
+			('foo:bar', 'page'),
+		):
+			#~ self.ui.open_url(href)
+			self.ui.mainwindow.pageview.do_link_clicked({'href': href})
+			msg = "Clicked: %s\nResulted in: %s" % (href, self.ui.mock_calls[-1])
+			if type == 'notebook':
+				self.assertTrue(self.ui.mock_calls[-1][0] == 'open_notebook', msg=msg)
+			elif type == 'page':
+				self.assertTrue(self.ui.mock_calls[-1][0] == 'open_page', msg=msg)
+			elif type == 'file':
+				self.assertTrue(self.ui.mock_calls[-1][0] == 'open_file', msg=msg)
+			elif type == 'mailto':
+				self.assertTrue(self.ui.mock_calls[-1][0:2] == ('open_with', 'email_client'), msg=msg)
+			else:
+				self.assertTrue(self.ui.mock_calls[-1][0:2] == ('open_with', 'web_browser'), msg=msg)
+			self.ui.mock_calls = [] # reset
+
+		# Some more tests that may not be covered above
+		for href, type in (
+			('zim+file://foo/bar?dus.txt', 'notebook'),
+			('file:///foo/bar', 'file'),
+			('mailto:foo@bar.com', 'mailto'),
+		):
+			self.ui.open_url(href)
+			msg = "open_url('%s')\nResulted in: %s" % (href, self.ui.mock_calls[-1])
+			if type == 'notebook':
+				self.assertTrue(self.ui.mock_calls[-1][0] == 'open_notebook', msg=msg)
+			elif type == 'file':
+				self.assertTrue(self.ui.mock_calls[-1][0] == 'open_file', msg=msg)
+			elif type == 'mailto':
+				self.assertTrue(self.ui.mock_calls[-1][0:2] == ('open_with', 'email_client'), msg=msg)
+			self.ui.mock_calls = [] # reset
+
+		# TODO test plugin with custom handler
+
 
 @tests.slowTest
 class TestNotebookDialog(tests.TestCase):
