@@ -23,6 +23,9 @@ class testPlugins(tests.TestCase):
 		self.assertTrue('spell' in plugins)
 		self.assertTrue('linkmap' in plugins)
 
+		# plugins listed here will be tested for is_profile_independent == True
+		profile_independent = ['automount',]
+
 		seen = {
 			'name': set(),
 			'description': set(),
@@ -70,6 +73,13 @@ class testPlugins(tests.TestCase):
 				self.assertTrue(isinstance(dep[i][1],bool))
 				self.assertTrue(isinstance(dep[i][2],bool))
 
+			# test is_profile_independent
+			self.assertTrue(isinstance(plugin.is_profile_independent,bool))
+			if name in profile_independent:
+				self.assertTrue(plugin.is_profile_independent)
+			else:
+				self.assertFalse(plugin.is_profile_independent)
+
 	def testDefaulPlugins(self):
 		'''Test loading default plugins'''
 		# Note that we use parent interface class here, so plugins
@@ -80,3 +90,5 @@ class testPlugins(tests.TestCase):
 		interface.uistate = zim.config.ConfigDict()
 		interface.load_plugins()
 		self.assertTrue(len(interface.plugins) > 3)
+
+	# TODO: create a full GtkUI object and load & unload each plugin in turn
