@@ -1,18 +1,16 @@
-
 # -*- coding: utf-8 -*-
 
-# Copyright 2008 Jaap Karssenberg <pardus@cpan.org>
+# Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 import tests
-from tests import TestCase
 
 import zim.plugins
 from zim.notebook import Path
 from zim.config import ConfigDict
 
-class TestPrintToBrowser(TestCase):
 
-	slowTest = True
+@tests.slowTest
+class TestPrintToBrowser(tests.TestCase):
 
 	def runTest(self):
 		'Test PrintToBrowser plugin'
@@ -30,11 +28,11 @@ class StubUI(object):
 	ui_type = 'stub'
 
 	def __init__(self):
-		self.notebook = tests.get_test_notebook()
+		self.notebook = tests.new_notebook()
 		self.page = self.notebook.get_page(Path('Test:foo'))
 		self.preferences = ConfigDict()
 		self.uistate = ConfigDict()
-	
+
 	def connect(*a): pass
 
 	def connect_after(*a): pass
