@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2008-2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''
 This module contains the base class for the zim application and the main
@@ -84,10 +84,10 @@ For asynchronous actions see L{zim.async}.
 
 
 # Bunch of meta data, used at least in the about dialog
-__version__ = '0.54'
+__version__ = '0.56'
 __url__='http://www.zim-wiki.org'
 __author__ = 'Jaap Karssenberg <jaap.karssenberg@gmail.com>'
-__copyright__ = 'Copyright 2008 - 2011 Jaap Karssenberg <jaap.karssenberg@gmail.com>'
+__copyright__ = 'Copyright 2008 - 2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>'
 __license__='''\
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -109,7 +109,8 @@ from getopt import gnu_getopt, GetoptError
 
 from zim.fs import File, Dir
 from zim.errors import Error
-from zim.config import data_dir, config_file, log_basedirs, ZIM_DATA_DIR
+from zim.config import data_dir, config_file, log_basedirs, ZIM_DATA_DIR, \
+					   XDG_CONFIG_HOME, ConfigDictFile
 
 
 logger = logging.getLogger('zim')
@@ -466,6 +467,23 @@ NOTE FOR BUG REPORTS:
 		module.main(None, *args)
 
 
+def ZimCmd():
+	'''Constructor to get a L{Application} object for zim itself
+	Use this object to spawn new instances of zim.
+	@returns: a L{Application} object for zim itself
+	'''
+	# Note that Application takes care of using sys.executable when
+	# needed
+	from zim.applications import Application
+	if ZIM_EXECUTABLE.endswith('.exe'):
+		return Application(ZIM_EXECUTABLE)
+	elif sys.executable:
+		# If not an compiled executable, we assume it is python
+		# (Application class does this automatically for python scripts
+		# ending in .py)
+		return Application((sys.executable, ZIM_EXECUTABLE))
+
+
 class NotebookInterface(gobject.GObject):
 	'''Base class for the application object
 
@@ -483,6 +501,9 @@ class NotebookInterface(gobject.GObject):
 	@signal: C{open-notebook (notebook)}:
 	Emitted to open a notebook in this interface
 
+	@signal: C{preferences-changed ()}:
+	Emitted when preferences have changed
+
 	@cvar ui_type: string to tell plugins what interface is supported
 	by this class. Currently this can be "gtk" or "html". If "ui_type"
 	is None we run without interface (e.g. commandline export).
@@ -499,6 +520,7 @@ class NotebookInterface(gobject.GObject):
 	# define signals we want to use - (closure type, return type and arg types)
 	__gsignals__ = {
 		'open-notebook': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+		'preferences-changed': (gobject.SIGNAL_RUN_LAST, None, ()),
 	}
 
 	ui_type = None
@@ -515,30 +537,39 @@ class NotebookInterface(gobject.GObject):
 		self.plugins = []
 
 		self.preferences = config_file('preferences.conf')
+		self.preferences['General'].setdefault('plugins',
+			['calendar', 'insertsymbol', 'printtobrowser', 'versioncontrol'])
+
 		self.uistate = None
 
+		self.load_early_plugins()
+
 		if not notebook is None:
 			self.open_notebook(notebook)
 
-	def load_plugins(self):
-		'''Loads all the plugins defined in the preferences
-
-		Typically called from the constructor of sub-classes.
+	def load_early_plugins(self):
+		'''Load all plugins that need to be loaded early
+		(For now these are notebook independent plugins only)
 		'''
-		default = ['calendar', 'insertsymbol', 'printtobrowser', 'versioncontrol']
-		self.preferences['General'].setdefault('plugins', default)
-		plugins = self.preferences['General']['plugins']
-		plugins = set(plugins) # Eliminate doubles
-
+		import zim.plugins
 		# Plugins should not have dependency on order of being added
 		# but sort them here to make behavior predictable.
+		plugins = self.preferences['General']['plugins']
+		for name in sorted(plugins):
+			klass = zim.plugins.get_plugin(name)
+			if klass.is_profile_independent:
+				self.load_plugin(name)
+
+	def load_plugins(self, independent_only=False):
+		'''Loads all the plugins defined in the preferences that are
+		not yet loaded.
+		'''
+		# Plugins should not have dependency on order of being added
+		# but sort them here to make behavior predictable.
+		plugins = self.preferences['General']['plugins']
 		for name in sorted(plugins):
 			self.load_plugin(name)
 
-		loaded = [p.plugin_key for p in self.plugins]
-		if set(loaded) != plugins:
-			self.preferences['General']['plugins'] = sorted(loaded)
-
 	def load_plugin(self, name):
 		'''Load a single plugin by name
 
@@ -570,6 +601,11 @@ class NotebookInterface(gobject.GObject):
 			plugin = klass(self)
 		except:
 			logger.exception('Failed to load plugin %s', name)
+			try:
+				self.preferences['General']['plugins'].remove(name)
+				self.preferences.set_modified(True)
+			except ValueError:
+				pass
 			return None
 		else:
 			self.plugins.append(plugin)
@@ -578,6 +614,7 @@ class NotebookInterface(gobject.GObject):
 		plugin.plugin_key = name
 		if not name in self.preferences['General']['plugins']:
 			self.preferences['General']['plugins'].append(name)
+			self.preferences.set_modified(True)
 
 		return plugin
 
@@ -591,8 +628,8 @@ class NotebookInterface(gobject.GObject):
 		'''
 		if isinstance(plugin, basestring):
 			name = plugin
-			assert name in [p.plugin_key for p in self.plugins]
-			plugin = filter(lambda p: p.plugin_key == name, self.plugins)[0]
+			plugin = self.get_plugin(name)
+			assert plugin is not None
 		else:
 			assert plugin in self.plugins
 			name = plugin.plugin_key
@@ -601,7 +638,52 @@ class NotebookInterface(gobject.GObject):
 		self.plugins.remove(plugin)
 		logger.debug('Unloaded plugin %s', name)
 
-		self.preferences['General']['plugins'].remove(name)
+		try:
+			self.preferences['General']['plugins'].remove(name)
+			self.preferences.set_modified(True)
+		except ValueError:
+			pass
+
+	def get_plugin(self, name):
+		'''Returns plugin object if this plugin is loaded, C{None}
+		otherwise.
+		'''
+		try:
+			return filter(lambda p: p.plugin_key == name, self.plugins)[0]
+		except IndexError:
+			return None
+
+	def save_preferences(self):
+		'''Save the preferences config file if modified
+		@emits: preferences-changed
+		'''
+		# For profile independent plugins, sync back to default
+		# preferences
+		if self.notebook and self.notebook.profile:
+			independent = []
+			for plugin in self.plugins:
+				if plugin.is_profile_independent:
+					independent.append(plugin.plugin_key)
+
+			if independent:
+				default = config_file('preferences.conf')
+				for name in independent:
+					if name not in default['General']['plugins']:
+						default['General']['plugins'].append(name)
+						default.set_modified(True)
+
+					section = plugin.__class__.__name__
+					if default[section] != plugin.preferences:
+						default[section].update(plugin.preferences)
+
+				if default.modified:
+					default.write()
+
+		# First emit, than write - avoid getting stuck with a set
+		# that crashes the application
+		if self.preferences.modified:
+			self.emit('preferences-changed')
+
 		self.preferences.write()
 
 	def open_notebook(self, notebook):
@@ -672,6 +754,61 @@ class NotebookInterface(gobject.GObject):
 			from zim.config import ConfigDict
 			self.uistate = ConfigDict()
 
+		if notebook.profile:
+			# the profile will determine what plugins to load
+			self.on_profile_changed(notebook)
+		else:
+			# load the rest of the plugins for the default prefences
+			self.load_plugins()
+
+		notebook.connect('profile-changed', self.on_profile_changed)
+
+	def on_profile_changed(self, notebook):
+		# Copy config for independent plugins
+		independent_preferences = {}
+		for plugin in self.plugins[:]:
+			if plugin.is_profile_independent:
+				independent_preferences[plugin.plugin_key] = \
+					plugin.preferences.copy()
+
+		# Switch config
+		if self.notebook.profile:
+			# Load the preferences for the profile
+			# In case new profile does not exist or is incomplete
+			# we cary over any settings from the current one
+			logger.debug('Profile changed to: %s', notebook.profile)
+			basename = self.notebook.profile.lower() + '.conf'
+			file = XDG_CONFIG_HOME.file(('zim', 'profiles', basename))
+			self.preferences.change_file(file)
+			self.preferences.write()
+		else:
+			# Load default preferences
+			# We do a full flush to reset to default
+			logger.debug('Profile reset to default')
+			preferences = config_file('preferences.conf')
+			file = preferences.file
+			self.preferences.change_file(file)
+			for section in self.preferences.values():
+				section.clear()
+			self.preferences.read() # HACK Forces reading default as well
+
+		# Notify ui objects
+		self.emit('preferences-changed')
+
+		# notify plugins of possible new preferences
+		# and remove old plugins
+		for plugin in self.plugins[:]:
+			if plugin.plugin_key in self.preferences['General']['plugins']:
+				plugin.emit('preferences-changed')
+			elif plugin.is_profile_independent:
+				self.preferences['General']['plugins'].append(plugin.plugin_key)
+				plugin.preferences.update(independent_preferences[plugin.plugin_key])
+			else:
+				self.unload_plugin(plugin)
+
+		# load new plugins
+		self.load_plugins()
+
 	def cmd_export(self, format='html', template=None, page=None, output=None, root_url=None, index_page=None):
 		'''Convenience method hat wraps L{zim.exporter.Exporter} for
 		commandline export
@@ -739,9 +876,8 @@ class NotebookInterface(gobject.GObject):
 
 		@todo: take this method outside this class
 		'''
-		from zim.applications import Application
-		zim = Application((ZIM_EXECUTABLE,) + args)
-		zim.spawn()
+		zim = ZimCmd()
+		zim.spawn(args=args)
 
 # Need to register classes defining gobject signals
 gobject.type_register(NotebookInterface)
