@@ -492,6 +492,16 @@ class ListDict(dict):
 					modified = self.modified
 					self.__setitem__(key, tuple(self[key]))
 					self.set_modified(modified) # don't change modified state
+				elif hasattr(klass, 'new_from_zim_config'):
+					# Class has special contructor
+					modified = self.modified
+					try:
+						self.__setitem__(key, klass.new_from_zim_config(self[key]))
+					except:
+						logger.exception(
+							'Invalid config value for %s: "%s"',
+							key, self[key])
+					self.set_modified(modified) # don't change modified state
 				else:
 					logger.warn(
 						'Invalid config value for %s: "%s" - should be of type %s',
@@ -637,13 +647,14 @@ class ConfigDict(ListDict):
 					section.append(ListDict())
 					section = section[-1]
 			elif '=' in line:
-				parameter, value = line.split('=', 1)
+				parameter, rawvalue = line.split('=', 1)
 				parameter = str(parameter.rstrip()) # no unicode
+				rawvalue = rawvalue.lstrip()
 				try:
-					value = self._decode_value(parameter, value.lstrip())
+					value = self._decode_value(parameter, rawvalue)
 					section[parameter] = value
 				except:
-					logger.warn('Failed to parse value for: %s', parameter)
+					logger.warn('Failed to parse value for key "%s": %s', parameter, rawvalue)
 			else:
 				logger.warn('Could not parse line: %s', line)
 
@@ -668,7 +679,7 @@ class ConfigDict(ListDict):
 				return value
 			except: pass
 
-			return json.loads('"%s"' % value.replace('"', '\\"')) # force string
+			return json.loads('"%s"' % value.replace('"', r'\"')) # force string
 
 	def dump(self):
 		'''Serialize the config to a "ini-style" config file.
@@ -733,7 +744,8 @@ class ConfigFile(ListDict):
 
 	def read(self):
 		'''Read data'''
-		# TODO: flush dict first ?
+		# No flush here - this is used by change_file()
+		# but may change in the future - so do not depend on it
 		try:
 			logger.debug('Loading %s', self.file.path)
 			self.parse(self.file.readlines())
@@ -759,6 +771,19 @@ class ConfigFile(ListDict):
 		self.set_modified(False)
 		return operation
 
+	def change_file(self, file, merge=True):
+		'''Change the underlaying file used to read/write data
+		Used to switch to a new config file without breaking existing
+		references to config sections.
+		@param file: a L{File} object for the new config
+		@param merge: if C{True} the new file will be read (if it exists)
+		and values in this dict will be updated.
+		'''
+		self.file = file
+		if file.exists():
+			self.read()
+		self.set_modified(True)
+
 
 class ConfigDictFile(ConfigFile, ConfigDict):
 	pass
