@@ -9,12 +9,10 @@ functionality, which works similar to the clipboard, but has a less
 straight forward API.
 '''
 
-# TODO support converting HTML to parsetree - need html Parser
-
 import gtk
 import logging
 
-from zim.fs import File
+from zim.fs import File, Dir
 from zim.notebook import Path
 from zim.parsing import is_url_re, url_encode, link_type, URL_ENCODE_READABLE
 from zim.formats import get_format, ParseTree, TreeBuilder
@@ -104,15 +102,74 @@ def unpack_urilist(text):
 	# FIXME be tolerant here for file://path/to/file uris here
 	text = text.strip('\x00') # Found trailing NULL character on windows
 	lines = text.splitlines() # takes care of \r\n
-	return [ line.decode('utf-8') 
+	return [ line.decode('utf-8')
 		for line in lines if line and not line.isspace() ]
 		# Just to be sure we also skip empty or whitespace lines
 
 
-def _link_tree(links, notebook, page):
+def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
+	'''Function to get a parsetree based on the selectiondata contents
+	if at all possible. Used by both copy-paste and drag-and-drop
+	methods.
+
+	The 'notebook' and optional 'path' arguments are used to format
+	links relative to the page which is the target for the pasting or
+	drop operation.
+
+	For image data, the parameters notebook and page are used
+	to save the image to the correct attachment folder and return a
+	parsetree with the correct image link.
+
+	@param selectiondata: a C{gtk.SelectionData} object
+	@param notebook: a L{Notebook} object
+	@param path: a L{Path} object
+
+	@returns: a L{ParseTree} or C{None}
+	'''
+	# TODO: check relative linking for all parsetrees !!!
+
+	targetname = str(selectiondata.target)
+	if targetname == PARSETREE_TARGET_NAME:
+		return ParseTree().fromstring(selectiondata.data)
+	elif targetname in (INTERNAL_PAGELIST_TARGET_NAME, PAGELIST_TARGET_NAME) \
+	or targetname in URI_TARGET_NAMES:
+		links = unpack_urilist(selectiondata.data)
+		return _link_tree(links, notebook, path)
+	elif targetname in TEXT_TARGET_NAMES:
+		# plain text parser should highlight urls etc.
+		# FIXME some apps drop text/uri-list as a text/plain mimetype
+		# try to catch this situation by a check here
+		text = selectiondata.get_text()
+		return get_format('plain').Parser().parse(text.decode('utf-8'))
+	elif targetname in IMAGE_TARGET_NAMES:
+		# save image
+		pixbuf = selectiondata.get_pixbuf()
+		if not pixbuf:
+			return None
+
+		dir = notebook.get_attachments_dir(path)
+		if not dir.exists():
+			logger.debug("Creating attachment dir: %s", dir)
+			dir.touch()
+
+		format, extension = _get_image_info(targetname)
+		if format is None:
+			format, extension = 'png', 'png' # default to png format
+
+		file = dir.new_file('pasted_image.%s' % extension)
+		logger.debug("Saving image from clipboard to %s", file)
+		pixbuf.save(file.path, format)
+
+		links = [file.uri]
+		return _link_tree(links, notebook, path)
+	else:
+		return None
+
+
+def _link_tree(links, notebook, path):
 	# Convert a list of links (of any type) into a parsetree
 	#~ print 'LINKS: ', links
-	#~ print 'NOTEBOOK and PAGE:', notebook, page
+	#~ print 'NOTEBOOK and PATH:', notebook, path
 	builder = TreeBuilder()
 	builder.start('zim-tree')
 	for i in range(len(links)):
@@ -132,7 +189,7 @@ def _link_tree(links, notebook, page):
 		logger.debug('Pasting link: %s (type: %s, isimage: %s)', link, type, isimage)
 
 		if isimage:
-			src = notebook.relative_filepath(file, page) or file.uri
+			src = notebook.relative_filepath(file, path) or file.uri
 			builder.start('img', {'src': src})
 			builder.end('img')
 		elif link.startswith('@'):
@@ -143,10 +200,10 @@ def _link_tree(links, notebook, page):
 		else:
 			if type == 'page':
 				href = Path(notebook.cleanup_pathname(link)) # Assume links are always absolute
-				link = notebook.relative_link(page, href) or link
+				link = notebook.relative_link(path, href) or link
 			elif type == 'file':
 				file = File(link) # Assume links are always URIs
-				link = notebook.relative_filepath(file, page) or file.uri
+				link = notebook.relative_filepath(file, path) or file.uri
 
 			builder.start('link', {'href': link})
 			builder.data(link)
@@ -154,7 +211,7 @@ def _link_tree(links, notebook, page):
 
 	builder.end('zim-tree')
 	tree = ParseTree(builder.close())
-	tree.resolve_images(notebook, page)
+	tree.resolve_images(notebook, path)
 	tree.decode_urls()
 	return tree
 
@@ -175,62 +232,292 @@ def _get_image_info(targetname):
 		return None, None
 
 
-def parsetree_from_selectiondata(selectiondata, notebook, page=None):
-	'''Function to get a parsetree based on the selectiondata contents
-	if at all possible. Used by both copy-paste and drag-and-drop
-	methods.
+class ClipboardItem(object):
+	'''Item that can be set on the clipboard'''
 
-	The 'notebook' and optional 'page' arguments are used to format
-	links relative to the page which is the target for the pasting or
-	drop operation.
+	def set(self, clipboard, clear_func):
+		'''Put this item on the clipboard
+		@param clipboard: a C{gtk.Clipboard} objet
+		@param clear_func: function to be passed to the clipboard as
+		callback on clearing the data
+		@implementation: must be implemented by sub-classes
+		'''
+		raise NotImplementedError
 
-	For image data, the parameters notebook and page are used
-	to save the image to the correct attachment folder and return a
-	parsetree with the correct image link.
+
+class TextItem(ClipboardItem):
+	'''Text item for the clipboard'''
+
+	def __init__(self, text):
+		self.text = text
+
+	def set(self, clipboard, clear_func):
+		clipboard.set_text(self.text)
+
+
+class UriItem(ClipboardItem):
+	'''Uri item for the clipboard'''
+
+	targets = URI_TARGETS + TEXT_TARGETS
+
+	def __init__(self, obj):
+		if isinstance(obj, basestring):
+			self.uri = obj
+			self.obj = None
+		else:
+			assert hasattr(obj, 'uri')
+			self.obj = obj
+			self.uri = obj.uri
+
+	def set(self, clipboard, clear_func):
+		clipboard.set_with_data(self.targets, self._get, clear_func)
+
+	def _get(self, clipboard, selectiondata, id, *a):
+		if id == URI_TARGET_ID:
+			selectiondata.set_uris((self.uri,))
+		else:
+			if isinstance(self.obj, (File, Dir)):
+				selectiondata.set_text(self.obj.user_path)
+			else:
+				selectiondata.set_text(self.uri)
+
+
+class ParseTreeItem(ClipboardItem):
+	'''Clipboard wrapper for a L{ParseTree}.'''
+
+	targets = (PARSETREE_TARGET,) + HTML_TARGETS + TEXT_TARGETS
+
+	def __init__(self, notebook, path, parsetree, format):
+		self.notebook = notebook # FIXME - should not need to keep this reference - resolve parsetree immediatly
+		self.path = path
+		self.parsetree = parsetree
+		self.format = format
+
+	def set(self, clipboard, clear_func):
+		clipboard.set_with_data(self.targets, self._get, clear_func, self) \
+			or logger.warn('Failed to set data on clipboard')
+
+	def _get(self, clipboard, selectiondata, id, *a):
+		'''Callback to get the data in a specific format
+		@param clipboard: a C{gtk.Clipboard} objet
+		@param selectiondata: a C{gtk.SelectionData} object to set the data on
+		@param id: target id for the requested data format
+		'''
+		logger.debug("Clipboard requests data as '%s', we have a parsetree", selectiondata.target)
+		if id == PARSETREE_TARGET_ID:
+			# TODO make links absolute (?)
+			xml = self.parsetree.tostring().encode('utf-8')
+			selectiondata.set(PARSETREE_TARGET_NAME, 8, xml)
+		elif id == HTML_TARGET_ID:
+			# FIXME - HACK - dump and parse as wiki first to work
+			# around glitches in pageview parsetree dumper
+			# main visibility when copy pasting bullet lists
+			# Same hack in print to browser plugin
+			dumper = get_format('wiki').Dumper()
+			text = ''.join( dumper.dump(self.parsetree) ).encode('utf-8')
+			parser = get_format('wiki').Parser()
+			parsetree = parser.parse(text)
+			#--
+			dumper = get_format('html').Dumper(
+				linker=StaticLinker('html', self.notebook, self.path) )
+			html = ''.join( dumper.dump(parsetree) )
+			html = wrap_html(html, target=selectiondata.target)
+			#~ print 'PASTING: >>>%s<<<' % html
+			selectiondata.set(selectiondata.target, 8, html)
+		elif id == TEXT_TARGET_ID:
+			logger.debug("Clipboard requested text, we provide '%s'" % self.format)
+			if self.format != 'wiki':
+				# FIXME - HACK - dump and parse as wiki first to work
+				# around glitches in pageview parsetree dumper
+				# main visibility when copy pasting bullet lists
+				# Same hack in print to browser plugin
+				dumper = get_format('wiki').Dumper()
+				text = ''.join( dumper.dump(self.parsetree) ).encode('utf-8')
+				parser = get_format('wiki').Parser()
+				parsetree = parser.parse(text)
+				#--
+			else:
+				parsetree = self.parsetree
+			#~ print ">>>>", self.format, parsetree.tostring()
+
+			if self.format in ('wiki', 'plain'):
+				dumper = get_format(self.format).Dumper()
+			else:
+				dumper = get_format(self.format).Dumper(
+					linker=StaticLinker(self.format, self.notebook, self.path) )
+
+			text = ''.join( dumper.dump(parsetree) ).encode('utf-8')
+			selectiondata.set_text(text)
+		else:
+			assert False, 'Unknown target id %i' % id
+
+
+class PageLinkItem(ClipboardItem):
+	'''Clipboard wrapper for a L{ParseTree}.'''
+
+	targets = (INTERNAL_PAGELIST_TARGET, PAGELIST_TARGET) + TEXT_TARGETS
+
+	def __init__(self, notebook, path):
+		self.notebookname = notebook.name
+		self.path = path
+
+	def set(self, clipboard, clear_func):
+		clipboard.set_with_data(self.targets, self._get, clear_func, self) \
+			or logger.warn('Failed to set data on clipboard')
+
+	def _get(self, clipboard, selectiondata, id, *a):
+		'''Callback to get the data in a specific format
+		@param clipboard: a C{gtk.Clipboard} objet
+		@param selectiondata: a C{gtk.SelectionData} object to set the data on
+		@param id: target id for the requested data format
+		'''
+		logger.debug("Clipboard requests data as '%s', we have a pagelink", selectiondata.target)
+		if id == INTERNAL_PAGELIST_TARGET_ID:
+			text = pack_urilist((self.path.name,))
+			selectiondata.set(INTERNAL_PAGELIST_TARGET_NAME, 8, text)
+		elif id == PAGELIST_TARGET_ID:
+			link = "%s?%s" % (self.notebookname, self.path.name)
+			text = pack_urilist((link,))
+			selectiondata.set(PAGELIST_TARGET_NAME, 8, text)
+		elif id == TEXT_TARGET_ID:
+			selectiondata.set_text(self.path.name)
+		else:
+			assert False, 'Unknown target id %i' % id
+
+
+class ClipboardManager(object):
+	'''Interface to the clipboard for copy and paste.
+	Replacement for gtk.Clipboard(), to be used everywhere in the
+	zim gui modules.
 	'''
-	targetname = str(selectiondata.target)
-	if targetname == PARSETREE_TARGET_NAME:
-		return ParseTree().fromstring(selectiondata.data)
-	elif targetname in (INTERNAL_PAGELIST_TARGET_NAME, PAGELIST_TARGET_NAME) \
-	or targetname in URI_TARGET_NAMES:
-		links = unpack_urilist(selectiondata.data)
-		return _link_tree(links, notebook, page)
-	elif targetname in TEXT_TARGET_NAMES:
-		# plain text parser should highlight urls etc.
-		# FIXME some apps drop text/uri-list as a text/plain mimetype
-		# try to catch this situation by a check here
-		text = selectiondata.get_text()
-		return get_format('plain').Parser().parse(text.decode('utf-8'))
-	elif targetname in IMAGE_TARGET_NAMES:
-		# save image
-		pixbuf = selectiondata.get_pixbuf()
-		if not pixbuf:
+
+	def __init__(self, atom):
+		'''Constructor
+		@param atom: clipboard name, can be either "CLIPBOARD" or "PRIMARY",
+		see C{gtk.Clipboard} for details.
+		'''
+		assert atom in ('CLIPBOARD', 'PRIMARY')
+		self.clipboard = gtk.Clipboard(selection=atom)
+		self.store = None
+		self._i_am_owner = False
+
+	def clear(self):
+		'''Clear clipboard if we set it'''
+		if self._i_am_owner:
+			self.clipboard.clear()
+
+	def set(self, item):
+		'''Set an item on the clipboard
+		@param item: a L{ClipboardItem}
+		'''
+		item.set(self.clipboard, self._clear)
+		self._i_am_owner = True
+		if self.store:
+			pass
+
+	def _clear(self, o, item):
+		self._i_am_owner = False
+		if self.store:
+			pass
+
+	def set_text(self, text):
+		'''Set text to the clipboard
+		@param text: text to set on the clipboard
+		@note: DO NOT USE THIS METHOD if you can use L{set_parsetree()}
+		instead
+		'''
+		item = TextItem(text)
+		self.set(item)
+
+	def get_text(self):
+		'''Get text from the clipboard.
+		@returns: (unicode) text or C{None}
+		@note: DO NOT USE THIS METHOD if you can use L{get_parsetree()}
+		instead
+		'''
+		text = self.clipboard.wait_for_text()
+		if isinstance(text, basestring):
+			text = text.decode('utf-8')
+		return text
+
+	def set_parsetree(self, notebook, path, parsetree, format='plain'):
+		'''Copy a parsetree to the clipboard. The parsetree can be pasted by
+		the user either as formatted text within zim or as plain text outside
+		zim. The tree can be the full tree for 'page', but also a selection.
+
+		@param notebook: the L{Notebook} object
+		@param path: the L{Path} object - used to resolve links etc.
+		@param parsetree: the actual L{ParseTree} to be set on the clipboard
+		@keyword format: the format to use for pasting text, e.g. 'wiki' or 'plain'
+		'''
+		item = ParseTreeItem(notebook, path, parsetree, format)
+		self.set(item)
+
+	def get_parsetree(self, notebook=None, path=None):
+		'''Get a parsetree from the clipboard.
+
+		Can handle various data types and convert them to L{ParseTree}
+		objects. So they can be pasted directly in a text buffer.
+
+		The 'notebook' and optional 'path' arguments are used to format
+		links relative to the page which is the target for the pasting or
+		drop operation. Otherwise absolute links will be used.
+
+		@param notebook: a L{Notebook} object
+		@param path: a L{Path} object
+
+		@returns: a L{ParseTree} or C{None}
+		'''
+		targets = self.clipboard.wait_for_targets()
+		logger.debug('Targets available for paste: %s, we want parsetree', targets)
+
+		if targets is None:
 			return None
 
-		dir = notebook.get_attachments_dir(page)
-		if not dir.exists():
-			logger.debug("Creating attachment dir: %s", dir)
-			dir.touch()
+		targets = [n for n in PARSETREE_ACCEPT_TARGET_NAMES if n in targets]
+			# Filter and sort by PARSETREE_ACCEPT_TARGET_NAMES
 
-		format, extension = _get_image_info(targetname)
-		if format is None:
-			format, extension = 'png', 'png' # default to png format
+		if targets:
+			name = targets[0]
+			logger.debug('Requesting data for %s', name)
+			selectiondata = self.clipboard.wait_for_contents(name)
+			if selectiondata:
+				return parsetree_from_selectiondata(selectiondata, notebook, path)
+			else:
+				logger.warn('Did not get requested data from clipboard')
+				return None
+		else:
+			logger.warn('Could not paste - no compatible data types on clipboard')
+			return None
 
-		file = dir.new_file('pasted_image.%s' % extension)
-		logger.debug("Saving image from clipboard to %s", file)
-		pixbuf.save(file.path, format)
+	def set_pagelink(self, notebook, path):
+		'''Copy a pagename to the clipboard. The pagename can be pasted by the
+		user either as a link within zim or as text outside zim.
+		@param notebook: a L{Notebook} object
+		@param path: a L{Path} object
+		'''
+		item = PageLinkItem(notebook, path)
+		self.set(item)
 
-		links = [file.uri]
-		return _link_tree(links, notebook, page)
-	else:
-		return None
+	def set_uri(self, uri):
+		'''Copy an uri to the clipboard
+		@param uri: an uri as string, or an object with an attribute C{uri}
+		'''
+		item = UriItem(uri)
+		self.set(item)
 
+Clipboard = ClipboardManager("CLIPBOARD") #: Singleton object for the default clipboard
+SelectionClipboard = ClipboardManager("PRIMARY") #: Singleton object for the selection clipboard (unix)
+
+
+
+
+########### Code to deal with HTML formatting on windows ############
 
 HTML_HEAD = '''\
 <meta http-equiv="Content-Type" content="text/html;charset=utf-8">
 <meta name="Description" content="Copy-Paste Buffer">
-<meta name="Generator" content="Zim">
-'''.strip()
+<meta name="Generator" content="Zim">'''
 
 def wrap_html(html, target):
 	'''Function to wrap html with appropriate headers based on target type'''
@@ -250,131 +537,6 @@ def wrap_html(html, target):
 ''' % (HTML_HEAD, html)
 
 
-class Clipboard(gtk.Clipboard):
-	'''This class extends the default gtk.Clipboard class with conversion
-	methods and convenience methods for zim specific data types. It's main
-	use is to get any paste data into a format that can be pasted into the
-	zim pageview widget.
-	'''
-
-	def set_parsetree(self, notebook, page, parsetree, format='plain'):
-		'''Copy a parsetree to the clipboard. The parsetree can be pasted by
-		the user either as formatted text within zim or as plain text outside
-		zim. The tree can be the full tree for 'page', but also a selection.
-
-		@param notebook: the L{Notebook} object
-		@param page: the L{Page} object - used to resolve links etc.
-		@param parsetree: the actual L{ParseTree} to be set on the clipboard
-		@keyword format: the format to use for pasting text, 'wiki' or 'plain'
-		'''
-		targets = [PARSETREE_TARGET]
-		targets.extend(HTML_TARGETS)
-		targets.extend(TEXT_TARGETS)
-		self.set_with_data(
-			targets,
-			Clipboard._get_parsetree_data, Clipboard._clear_data,
-			(notebook, page, parsetree, format)
-		) or logger.warn('Failed to set data on clipboard')
-
-	def _get_parsetree_data(self, selectiondata, id, data):
-		logger.debug("Clipboard data request of type '%s', we have a parsetree", selectiondata.target)
-		notebook, page, parsetree, format = data
-		if id == PARSETREE_TARGET_ID:
-			xml = parsetree.tostring().encode('utf-8')
-			selectiondata.set(PARSETREE_TARGET_NAME, 8, xml)
-		elif id == HTML_TARGET_ID:
-			# FIXME - HACK - dump and parse as wiki first to work
-			# around glitches in pageview parsetree dumper
-			# main visibility when copy pasting bullet lists
-			# Same hack in print to browser plugin
-			dumper = get_format('wiki').Dumper()
-			text = ''.join( dumper.dump(parsetree) ).encode('utf-8')
-			parser = get_format('wiki').Parser()
-			parsetree = parser.parse(text)
-			#--
-			dumper = get_format('html').Dumper(
-				linker=StaticLinker('html', notebook, page) )
-			html = ''.join( dumper.dump(parsetree) )
-			html = wrap_html(html, target=selectiondata.target)
-			#~ print 'PASTING: >>>%s<<<' % html
-			selectiondata.set(selectiondata.target, 8, html)
-		elif id == TEXT_TARGET_ID:
-			dumper = get_format(format).Dumper()
-			text = ''.join( dumper.dump(parsetree) ).encode('utf-8')
-			selectiondata.set_text(text)
-		else:
-			assert False, 'Unknown target id %i' % id
-
-	def request_parsetree(self, callback, notebook, page=None, block=False):
-		'''Request a parsetree from the clipboard if possible. Because pasting
-		is asynchronous a callback needs to be provided to accept the parsetree.
-		This callback just gets the parsetree as single argument.
-
-		For image data, the parameters notebook and page are used
-		to save the image to the correct attachment folder and return a
-		parsetree with the correct image link.
-
-		If 'block' is True the operation is blocking. This is only intended
-		for testing and should not be used for real functionality.
-		'''
-		targets = self.wait_for_targets()
-		logger.debug('Targets available for paste: %s, we want parsetree', targets)
-
-		if targets is None:
-			return
-
-		for name in PARSETREE_ACCEPT_TARGET_NAMES:
-			if name in targets:
-				break
-		else:
-			name = None
-
-		def request_parsetree_data(self, selectiondata, data):
-			tree = parsetree_from_selectiondata(selectiondata, notebook, page)
-			callback(tree)
-
-		if name:
-			logger.debug('Requesting data for %s', name)
-			if block:
-				selectiondata = self.wait_for_contents(name)
-				return parsetree_from_selectiondata(selectiondata, notebook, page)
-			else:
-				self.request_contents(name, request_parsetree_data)
-		else:
-			logger.warn('Could not paste - no compatible data types on clipboard')
-
-	def set_pagelink(self, notebook, page):
-		'''Copy a pagename to the clipboard. The pagename can be pasted by the
-		user either as a link within zim or as text outside zim.
-		'''
-		targets = [INTERNAL_PAGELIST_TARGET, PAGELIST_TARGET]
-		targets.extend(gtk.target_list_add_text_targets(info=TEXT_TARGET_ID))
-		self.set_with_data(
-			targets,
-			Clipboard._get_pagelink_data, Clipboard._clear_data,
-			(notebook.name, page.name)
-		) or logger.warn('Failed to set data on clipboard')
-
-	def _get_pagelink_data(self, selectiondata, id, data):
-		logger.debug("Clipboard data request of type '%s', we have pagelink", selectiondata.target)
-		notebookname, pagename = data
-		if id == INTERNAL_PAGELIST_TARGET_ID:
-			text = pack_urilist((pagename,))
-			selectiondata.set(INTERNAL_PAGELIST_TARGET_NAME, 8, text)
-		elif id == PAGELIST_TARGET_ID:
-			link = "%s?%s" % (notebookname, pagename)
-			text = pack_urilist((link,))
-			selectiondata.set(PAGELIST_TARGET_NAME, 8, text)
-		elif id == TEXT_TARGET_ID:
-			selectiondata.set_text(pagename)
-		else:
-			assert False, 'Unknown target id %i' % id
-
-	def _clear_data(self, data):
-		# Callback to clear our clipboard data - pass because we keep no state
-		pass
-
-
 class Win32HtmlFormat:
 	'''This class adds support for Windows "HTML Format" clipboard content type
 
