@@ -17,6 +17,7 @@ import logging
 from zim.index import IndexPath
 from zim.notebook import Path
 from zim.gui.widgets import ui_environment, BrowserTreeView, \
+	populate_popup_add_separator, \
 	ErrorDialog
 from zim.gui.clipboard import \
 	Clipboard, \
@@ -565,13 +566,23 @@ class PageTreeView(BrowserTreeView):
 			or BrowserTreeView.do_key_press_event(self, event)
 
 	def do_initialize_popup(self, menu):
-		self.ui.populate_popup('page_popup', menu)
+		# TODO get path first and determine what menu options are valid
+		path = self.get_selected_path() or Path(':')
+		self.ui.populate_popup('page_popup', menu, path)
+
+		populate_popup_add_separator(menu)
+		item = gtk.ImageMenuItem('gtk-copy')
+		item.connect('activate', lambda o: self.do_copy())
+		menu.append(item)
+		menu.show_all()
+
+		self.populate_popup_expand_collapse(menu)
 
 	def do_copy(self):
 		#~ print '!! copy location'
 		page = self.get_selected_path()
 		if page:
-			Clipboard().set_pagelink(self.ui.notebook, page)
+			Clipboard.set_pagelink(self.ui.notebook, page)
 
 	def do_drag_data_get(self, dragcontext, selectiondata, info, time):
 		assert selectiondata.target == INTERNAL_PAGELIST_TARGET_NAME
@@ -632,8 +643,6 @@ class PageTreeView(BrowserTreeView):
 		if model is None:
 			return None # index not yet initialized ...
 
-		model.select_page(path)
-
 		treepath = model.get_treepath(path)
 		if treepath:
 			# path existed, now select it
@@ -660,6 +669,8 @@ class PageTreeView(BrowserTreeView):
 
 		self._cleanup = rowreference
 
+		model.select_page(path) # highlight in model
+
 		return treepath
 
 	def select_treepath(self, treepath):
@@ -713,7 +724,10 @@ class PageIndex(gtk.ScrolledWindow):
 		self.treeview.connect('insert-link',
 			lambda v, p: self.ui.mainwindow.pageview.insert_links([p]))
 
-		ui.connect('open-notebook', self.on_open_notebook)
+		if self.ui.notebook:
+			self.on_open_notebook(self.ui, self.ui.notebook)
+		else:
+			ui.connect('open-notebook', self.on_open_notebook)
 		ui.connect('open-page', self.on_open_page)
 		ui.connect('start-index-update', lambda o: self.disconnect_model())
 		ui.connect('end-index-update', lambda o: self.reload_model())
