@@ -246,7 +246,8 @@ class PluginClass(gobject.GObject):
 	will be initialized when the plugin is loaded.
 
 	Plugin classes should define two class attributes: L{plugin_info} and
-	L{plugin_preferences}.
+	L{plugin_preferences}. Optionally, they can also define the class
+	attribute L{is_profile_independent}.
 
 	@cvar plugin_info: A dict with basic information about the plugin,
 	it should contain at least the following keys:
@@ -275,6 +276,13 @@ class PluginClass(gobject.GObject):
 	Changes to these preferences will be stored in a config file so
 	they are persistent.
 
+	@cvar is_profile_independent: A boolean indicating that the plugin
+	configuration is global and not meant to change between notebooks.
+	The default value (if undefined) is False. Plugins that set
+	L{is_profile_independent} to True will be initialized before
+	opening the notebook. All other plugins will only be loaded after
+	the notebook is initialized.
+
 	@ivar ui: the main application object, e.g. an instance of
 	L{zim.gui.GtkInterface} or L{zim.www.WWWInterface}
 	@ivar preferences: a C{ListDict()} with plugin preferences
@@ -304,6 +312,8 @@ class PluginClass(gobject.GObject):
 
 	plugin_preferences = ()
 
+	is_profile_independent = False
+
 	@classmethod
 	def check_dependencies_ok(klass):
 		'''Checks minimum dependencies are met
@@ -339,6 +349,7 @@ class PluginClass(gobject.GObject):
 		plugin in the methods L{initialize_ui()}, L{initialize_notebook()},
 		L{finalize_ui()} and L{finalize_notebook()} where apropriate.
 		'''
+		# NOTE: this method is decorated by the meta class
 		gobject.GObject.__init__(self)
 		self.ui = ui
 		assert 'name' in self.plugin_info, 'Plugins should provide a name in the info dict'
@@ -426,6 +437,7 @@ class PluginClass(gobject.GObject):
 
 		@implementation: optional, may be implemented by subclasses.
 		'''
+		# NOTE: this method is decorated by the meta class
 		pass
 
 	def finalize_ui(self, ui):
